/*
 * $Id: ClientHandler.java,v 1.3 1998/04/06 09:16:51 vmallet Exp $
 *
 *
 * $Log: ClientHandler.java,v $
 * Revision 1.3  1998/04/06 09:16:51  vmallet
 * Changed addNewPlayer() to addPlayer().
 * Added the setupPlayer() method call in doJoinGame().
 *
 * Revision 1.2  1998/03/11 02:16:19  vmallet
 * Did a lot of requests handling code.
 * Removed the "removePlayerFromGame()" method.
 *
 * Revision 1.1  1998/03/02 19:14:24  vmallet
 * Initial revision
 *
 *
 */

package game;

import java.io.*; 
import java.net.*; 
import java.util.*; 

/**
 * The <code>ClientHandler</code> class handles the client connection
 * in the server. This class is intended to be used in the <i>jags</i>
 * server.
 * [DESCRIBE ME MORE]
 * 
 * @version $Revision: 1.3 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public class ClientHandler extends Handler {

  /**
   * The server that created this ClientHandler.
   */  
  GameServer server;
  
  /**
   * Creates a new ClientHandler.
   *
   * @param server the server wishing to handle the client.
   * @param socket the socket which the client is connected to.
   */
  public ClientHandler(GameServer server, Socket socket) 
    throws SocketException {   
    super(socket);
    this.server = server;
  }
  
  /**
   * run! run! [DESCRIBE ME]
   */  
  public void run() {
    if (debug)
      System.out.println("<ClientHandler> run() - hmmm connected: " 
			 + gameLink.isConnected());
    while (gameLink.isConnected()) {
      try { sleep(1000); } catch (InterruptedException e) { }
    }
    
    if (debug)
      System.out.println("<ClientHandler> run() - Lost Client Connection!");

    lostConnectionNotify();

    //@@ hu ? Yope. it's not so bad after all.
    server.removeHandler(this);
    
    // remove all managed players from all games players are in.
    for (Enumeration e = getManagedPlayers(); e.hasMoreElements(); ) {
      Player player = (Player) e.nextElement();
      for (Enumeration f = getPlayerGames(player); f.hasMoreElements(); ) {
	GameHandler handler = (GameHandler) f.nextElement();
	server.removePlayerFromGame(player, handler);
      }
    }
  } // run

  /**
   * Messaged when a new data frame is received by the <code>GameLink</code>
   * object and this frame belongs to the groups the GameLinkListener
   * asked to listen to.
   *
   * @param frame the frame that has just been received.
   *
   * @see GameLinkListener
   */  
  public void frameReceived(Object link, LinkFrame frame) {
    int request = frame.getRequest();

    LinkFrame answer = new LinkFrame();    
    answer.setGameId(HANDLER_ID);
    answer.setGroup(HANDLER_GROUP);

    switch (request) {
    case RQ_SERVERID:
      if (debug)
	System.out.println("<ClientHandler> frameReceived() - got RQ_SERVERID");
      doServerId(frame);
      break;
      
    case RQ_JOIN_GAME:
      if (debug)
	System.out.println("<ClientHandler> frameReceived() - got RQ_JOIN_GAME");
      doJoinGame(frame);
      break;

    case RQ_LEAVE_GAME: 
      if (debug)
	System.out.println("<ClientHandler> frameReceived() - got RQ_LEAVE_GAME");
      doLeaveGame(frame);
      break;
      
    default:
      answer.setData("I'm the Server. You'll play later!");
      gameLink.sendBlindly(answer);
      break;
    }
  } // frameReceived


  /** 
   * Handles the RQ_SERVERID request.
   * 
   * @param frame the frame containing the RQ_SERVERID request.
   */  
  void doServerId(LinkFrame frame) {
    LinkFrame answer = new LinkFrame();
    answer.setGameId(HANDLER_ID);
    answer.setGroup(HANDLER_GROUP);

    Object[] data = new Object[5];
    data[0] = new Integer(0);  //@@ server version. CHECK THIS (property?)
    data[1] = server.getName();         // server name.
    data[2] = server.getAllGameInfos(); // info about new and running games
    answer.setRequest(RQ_SERVERID, NONE);
    answer.setData(data);
    gameLink.sendBlindly(answer);
  }
    
  /**
   * Handles RQ_JOIN_GAME request. 
   * 
   * @param frame the frame containing the RQ_JOIN_GAME request.
   */  
  void doJoinGame(LinkFrame frame) {
    LinkFrame answer = new LinkFrame();
    answer.setGameId(HANDLER_ID);
    answer.setGroup(HANDLER_GROUP);

    Object[] recvData;
    //@@ We should proctect against exceptions.
    recvData          = (Object[]) frame.getData();
    Object id         = recvData[0]; // requested game Id.
    String playerName = (String) recvData[1]; // player name.
    System.out.println("   +gameId: " + id);
    System.out.println("   +playername: " + (String) recvData[1]);
    
    GameHandler handler = server.getGame(id);
    
    if (handler == null) {
      answer.setRequest(REPLY_INVALID_GAME_ID, NONE);
      answer.setData(id);
      gameLink.sendBlindly(answer);
      
      return;
    }

    // create a new player and try to add it to the game.
    Player player = null;
    
    try {
      player = handler.getGame().createPlayer(playerName, 
			  new GameLinkFilter(handler.getGame().getId(), gameLink));
      handler.getGame().addPlayer(player);
    } catch (TooManyPlayersException e) {
      answer.setRequest(REPLY_TOO_MANY_PLAYERS, NONE);
      answer.setData(new Integer(handler.getPlayerCount()));
      gameLink.sendBlindly(answer);
      return;
    } catch (ClosedGameException e) {
      answer.setRequest(REPLY_CLOSED_GAME, NONE);
      gameLink.sendBlindly(answer);
      return;
    }

    // Here the new player has been added to the game.
    //@@ notify clients that the corresponding GameInfo chanfed.
    //@@ should we do that somewhere else ? Hmm I think so.. -vm980301
    
    System.out.println("<ClientHandler> Players: " + handler.getPlayerCount());

    answer.setRequest(ST_INFO_CHANGED, NONE);
    answer.setData(new GameInfo(handler));
    server.sendToClients(answer);

    addManagedPlayer(player, handler);

    // do whatever is necessary to create client mirroring game.

    //@@ hmmm, my first try at doing that...
    answer.setRequest(ST_CREATE_GAME, NONE);
    Object[] data = new Object[5];
    data[0]       = handler.getGame().getClientGameClassName();
    data[1]       = handler.getGame().getId();
    data[2]       = player.getName();
    data[3]       = new Integer(player.getId());
    data[4]       = handler.getGame().getName();
    answer.setData(data);
    
    gameLink.sendBlindly(answer);

    handler.getGame().setupPlayer(player);

  } // doJoinGame
  

  /** 
   * Handles the RQ_LEAVE_GAME request.
   * 
   * @param frame the frame containing the RQ_LEAVE_GAME request.
   */  
  void doLeaveGame(LinkFrame frame) {
    LinkFrame answer = new LinkFrame();
    answer.setGameId(HANDLER_ID);
    answer.setGroup(HANDLER_GROUP);

//     Object[] data = new Object[5];
//     data[0] = new Integer(0);  //@@ server version. CHECK THIS (property?)
//     data[1] = server.getName();         // server name.
//     data[2] = server.getAllGameInfos(); // info about new and running games
//     answer.setRequest(RQ_SERVERID, NONE);
//     answer.setData(data);
//     gameLink.sendBlindly(answer);
  } // doLeaveGame

  



} // ClientHandler
