/*
 * $Id: Game.java,v 1.3 1998/04/07 12:20:02 vmallet Exp $
 *
 *
 * $Log: Game.java,v $
 * Revision 1.3  1998/04/07 12:20:02  vmallet
 * First use of the getFrameData() for player creation.
 * Added some player removing code.
 *
 * Revision 1.2  1998/04/06 09:25:45  vmallet
 * Changed addNewPlayer() to addPlayer().
 * Added setupPlayer(), getPlayers().
 * Other minor changes.
 *
 * Revision 1.1  1998/03/11 02:05:53  vmallet
 * Initial revision
 *
 *
 */

package game;

import java.io.*;
import java.net.*;
import java.util.*;


/**
 * The <code>Game</code> class is the abstract class specifying what 
 * features a game that will be handled by the Jags server needs to provide.
 * Such a game will inherit from <code>Game</code> and implement all the
 * abstract methods, and if necessary override some of the concrete
 * methods. The <code>Game</code> class also provides basic functonalities
 * common to all games, such as handling the observers, adding and removing
 * players, loading the properties, etc.
 * 
 * @version $Revision: 1.3 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public abstract class Game implements GameLinkListener, GameLinkConstants {

  /**
   * Name of the Game
   */
  protected String name;

  /**
   * Maximum nomber of players
   */
  protected int maxPlayers;

  /**
   * Name of the class that has to be used in the Jags client
   * to be able to connect to this game in the Jags server.
   */
  protected String clientGameClassName;

  /**
   * Properties of the game, dynamically loaded at game creation.
   */
  protected Properties props = new Properties();

  /**
   * Id of this game.
   */
  protected Object id;

  /**
   * Server in which the game has been created.
   */
  protected GameServer server;

  /**
   * Set of observers that will be notified of game changes, typically the
   * <code>Player</code> objects.
   */
  protected Observable observers = new Observable() {
    public void notifyObservers(Object o) {
      setChanged();
      super.notifyObservers(o);
    }
  };

  /**
   * if true, enables the printing of debug informations on the console.
   */  
  protected boolean debug = true;



  //== constructors =======================================================

  /**
   * Creates a new game [DESCRIBE ME MORE]
   */
  public Game() {
    String className = getClass().getName();
    String propName  = className.substring(className.lastIndexOf('.') + 1);
    URL    url       = getClass().getResource(propName + ".properties");
    
    try {
      props.load(url.openStream());
    } catch (IOException e) {
      if (debug)
	System.out.println("<Game> <init>() - Error loading props");
      //@@ do something here, ie load defaults if applicable.
    }

    //@@ remove this:
//     if (debug) {
//       System.out.println("<Game> props= " + props);
//       for (Enumeration e = props.propertyNames(); e.hasMoreElements(); )
// 	System.out.println("<Game> key = '" + (String) e.nextElement() + "'");
//     }
    
    name       = props.getProperty("game.name", "unnamed");
    System.out.println("<Game> init: name=" + name);
    maxPlayers = Integer.parseInt(props.getProperty("game.maxPlayers", "0"));
    clientGameClassName = props.getProperty("game.clientClassName", "none");
  }



  //== methods ===========================================================

  /**
   * Returns the maximum number of players in that game.
   */ 
  int getMaximumPlayers() {
    return maxPlayers;
  } 

  /**
   * Returns the name of this game.
   */
  public String getName() {
    return name;
  }

  /**
   * Returns the name of the class that has to be used in the Jags client
   * to be able to connect to this game in the Jags server.
   */ 
  public String getClientGameClassName() {
    return clientGameClassName;
  }

  /**
   * Sets the server in which the game has been created.
   */
  public void setServer(GameServer server) {
    this.server = server;
  }


  /**
   * Creates a new particular <code>Player</code> for this game. This method 
   * has to be overriden by children so that the created player is of the right 
   * type.
   *
   * @param name the name of the new player to create
   * @param link the <code>GameLinkFilter</code> object that represents the link
   *             between the server and the client the player comes from.
   * @return the newly created player of the right type to be used in this
   *         game.
   */
  public abstract Player createPlayer(String name, GameLinkFilter link);


  /**
   * Adds the specified player in this game, so that this player is able
   * to play the game.
   *
   * @param player the player to add in this game.
   *
   * @exception TooManyPlayersException if the maximum number of players
   *            allowed in this game has already been reached.
   * @exception ClosedGameException if the game is closed (ie private).
   *
   * @see #removePlayer
    */
  public void addPlayer(Player player) 
    throws TooManyPlayersException, ClosedGameException {

    if (debug)
      System.out.println("<Game> addPlayer(): adding " + player
		         + " to game " + getName());
    
    if (getPlayerCount() >= getMaximumPlayers())
      throw new TooManyPlayersException("Max is " + getMaximumPlayers());

    GameLinkFilter link = player.getGameLink();
    synchronized (players) {
      players.addElement(player);
      linkPlayers.put(link, player);
      link.addGameLinkListener(this, GAME_GROUP);
    }

    addObserver(player);

  }


  /**
   * [DESCRIBE ME]
   */  
  public void setupPlayer(Player player) {


    
    //@@ -980404 new code for creating the game in the client
    //@@         of the given player
    LinkFrame frame = new LinkFrame();
    
    frame.setGroup(GAME_GROUP);
    frame.setRequest(ST_CREATE_PLAYER, player.getId());
    frame.setData(player.getFrameData());

    notifyObservers(frame);

    // Now, create all game components in new client.
    
    //@@ should use some sort of Player.getFrameData() method (see with grishna)
    
    for (Enumeration e = getPlayers(); e.hasMoreElements(); ) {
      Player p = (Player) e.nextElement();
      if (p.getId() != player.getId()) {
	frame.setSubRequest(p.getId());
	frame.setData(p.getFrameData());
// 	data[0] = p.getName();
// 	data[1] = new Integer(p.getId());
// 	frame.setData(data);
	
	player.addObserver(p);
	player.getGameLink().sendBlindly(frame);
      }
    }

    for (Enumeration e = getPlayers(); e.hasMoreElements(); ) {
      Player p = (Player) e.nextElement();
      p.addObserver(player);
    }
  }


  /**
   * Removes the specified player from this game. The player won't be able
   * to play anymore, unless it is added back to the game.
   *
   * @param player the player that has to be removed from this game.
   *
   * @see #addPlayer
   */
  public void removePlayer(Player player) {
    if (debug)
      System.out.println("<Game> removePlayer(): removing " + player
			 + " from game " + getName());


    deleteObserver(player);

    GameLinkFilter link = player.getGameLink();
    link.deleteGameLinkListener(this, GAME_GROUP);
    synchronized (players) {
      players.removeElement(player);
      linkPlayers.remove(link);
    }

    LinkFrame frame = new LinkFrame();
    frame.setGroup(GAME_GROUP);
    frame.setRequest(ST_REMOVE_PLAYER, player.getId());
    notifyObservers(frame);

    for (Enumeration e = getPlayers(); e.hasMoreElements(); ) {
      Player p = (Player) e.nextElement();
      p.deleteObserver(player);
    }

    player.deleteObserver(player);
  }
  

  /**
   * Returns the number of players currently playing this game.
   */
  public int getPlayerCount() {
    synchronized (players) {
      return players.size();
    }
  }


  /**
   * Returns an enumeration of players in this game. To avoid synchronization 
   * problems, the array of players is first duplicated, then returned as an
   * enumeration.
   */
  public Enumeration getPlayers() {
    Vector playersCopy;

    synchronized (players) {
      playersCopy = (Vector) players.clone();
    }

    return playersCopy.elements();
  }


  /**
   * Returns whether this game has already ended or if it is still running.
   * By default, a game is terminated when there are no more players, but
   * this method should be overriden by children. 
   * <p>
   * [CHECK ME. A newly created game also has 0 player ! -vm980301]
   *
   * @return true if the game is "dead", false otherwise.
   */
  public boolean isTerminated() {
    //@@ check me ! -vm980301
    if (debug)
      System.out.println("<Game> getPlayerCount() == " + getPlayerCount());
    return getPlayerCount() == 0; 
  }
 
  /**
   * Starts the Game [CHECK: DO WE NEED THIS?]
   */
  public abstract void start();
  
  
  /**
   * Returns the id of the game.
   */
  public Object getId() {
    return id;
  }
  
  /**
   * Sets the id of the game.
   */
  public void setId(Object id) {
    this.id = id;
  }

  // == implements GameLinkListener ====================================

  /**
   * Messaged when a new data frame is received by the <code>GameLinkFilter</code>
   * object and this frame belongs to the groups the GameLinkListener
   * asked to listen to.
   *
   * @param link  the GameLink that actually received the frame.
   * @param frame the frame that has just been received.
   *
   * @see GameLink
   * @see GameLinkListener#frameReceived
   */  
  public void frameReceived(Object link, LinkFrame frame) {
    System.out.println("<Game> frameReceived()");
    
    int request = frame.getRequest();
    
    switch (request) {
    case RQ_LEAVE_GAME:
      Player player = (Player) linkPlayers.get(link);
      if (player == null) {
	if (debug)
	  System.out.println("<Game> LEAVE_GAME: unmatched player !");
	return;
      }
      if (player.getId() == frame.getSubRequest()) {
	//@@ removePlayer(player);
	//@@ have the server do this for us
	server.removePlayerFromGame(player, server.getGame(getId()));
      }
      break;
      
    default:
      if (debug)
	System.out.println("<Game> frameReceived: unhandled request: " + request);
      break;
    }
  }





  //== Observers handling code ======================================
  

  /**
   * Adds an observer to the set of observers for this game.
   * @param obs an observer to be added.
   */
  public void addObserver(Observer obs) {
    observers.addObserver(obs);
  }

  /**
   * Deletes an observer from the set of observers for this game.
   * @param obs the observer to be deleted
   */
  public void deleteObserver(Observer obs) {
    observers.deleteObserver(obs);
  }

  /**
   * Clears the observerlist so that this game no longer has any observers.
   */
  public void deleteObservers() {
    observers.deleteObservers();
  }

  /**
   * Returns the number of observers observing this game.
   * @return the number of observers observing this game.
   */  
  public int countObservers() {
    return observers.countObservers();
  }

  /**
   * Notifies all observers that this game has changed, giving them
   * the frame containing the changes.
   *
   * @param frame the frame containing the changes
   */
  public void notifyObservers(LinkFrame frame) {
    System.out.println("<Game> notifyObservers: obs=" + observers.countObservers());
    observers.notifyObservers(frame);
  }


  // == Private data =======================================
  
  
  
  /**
   * Set of players playing in this game.
   */
  protected Vector players = new Vector();

  /**
   * Set of players indexed on their <code>GameLinkFilter</code> objects.
   */
  protected Hashtable linkPlayers = new Hashtable();
  
}

