/*
 * $Id: GameFactory.java,v 1.1 1998/03/02 19:28:09 vmallet Exp $
 *
 *
 * $Log: GameFactory.java,v $
 * Revision 1.1  1998/03/02 19:28:09  vmallet
 * Initial revision
 *
 *
 */

package game;

import java.util.*;
import java.net.*;


/**
 * GameFactory.java
 * 
 * @version $Revision: 1.1 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public class GameFactory {

  public boolean debug = true;

  /**
   * Cache for loaded classes
   */  
  Hashtable  classTable = new Hashtable();

  /**
   * Properties loaded from file at creation.
   */  
  Properties props;

  /**
   * Shared instance of the Factory.
   */  
  static GameFactory sharedInstance;
  
  
  /**
   * Creates a new GameFactory.
   */
  
  protected GameFactory() {
    props = new Properties();
    //    debug = Boolean.getBoolean("html.aural.debug");

    URL url = null;
    try {
      url = getClass().getResource("GameFactory.properties");
      props.load(url.openStream());
    } catch (Exception e) {
      if (debug)
	System.out.println("<GameFactory> warning: " + 
			   "failed to load properties: " + url);
      // we'll use defaults
    }
    
    // Load all game classes here
    //@@ 98.02.22 we could may manage that a different way, such as
    //@@ 98.02.22 dynamic class loading etc.

    loadClasses();
  }


  /**
   * Load all game classes into the server, so they are ready
   * to be used.
   */  
  public void loadClasses() {
    for (Enumeration e = props.propertyNames(); e.hasMoreElements(); ) {
      String prop = (String) e.nextElement();
      System.out.println("prop: " + prop);
      if (prop.substring(0, 10).equals("game.game.")) {
	String className = prop.substring(10);
	System.out.println("Loading class " + className);
	getGameClass(className);
      }
    }
  }
  

  /**
   * Load a Game class according to the specified game name.
   * The class corresponding to the name is determined by searching
   * in the properties.
   *
   * @param name the name of the game.
   */
  public Class getGameClass(String name) {
    Class gameClass = null;
    
    try {
      if((gameClass = (Class) classTable.get(name)) == null) {
	String gameClassName = props.getProperty("game.game." + name, 
						 "game.game." + name);
	System.out.println("gameClassName=" + gameClassName);
	gameClass = Class.forName(gameClassName);
	
	if(gameClass != null) 
	  classTable.put(name, gameClass);
      }
    }
    catch(Exception e) {
      if (debug)
	System.out.println("<GameFactory> getGameClass(): " + 
			   "failed to load class for <" +
			   name + ">. ");
      //@@ 98.02.22 should throw some sort of exception here
    }

    return gameClass;
  }



  /**
   * Creates a new instance of the specified game.
   *
   * @param name the name of the game.
   * @return a new instance of the <code>Game</code> object
   *         corresponding to the given name.
   */
  public Game createGame(String name) {  
    
    Class gameClass;
    Game  game = null;
    
    gameClass = getGameClass(name);
    
    //@@ 98.02.22 here we should handle exceptions at the getGameClass
    //@@ 98.02.22 level instead of testing the returned value.
    try {
      if (gameClass != null)
	game = (Game) gameClass.newInstance();
    } catch (InstantiationException e) {
      game = null; //@@ 98.02.22 maybe do something better here
    } catch (IllegalAccessException e) {
      game = null; //@@ 98.02.22 maybe do something better here
    }
    
    if (game != null)
      game.setId(name);

    return game;
  }


  /**
   * Returns the shared instance of the factory.
   */
  public static synchronized GameFactory getSharedInstance() {
    if (sharedInstance == null)
      sharedInstance = new GameFactory();

    return sharedInstance;
  }

  /**
   * 
   */  
  public Enumeration getGameInfos() {
    return new GameFactoryEnumerator(classTable.keys());
  }


}



final
class GameFactoryEnumerator implements Enumeration {
  Enumeration classes;
  
  GameFactoryEnumerator(Enumeration e) {
    classes = e;
  }
  
  public boolean hasMoreElements() {
    return classes.hasMoreElements();
  }
  
  public Object nextElement() {
    synchronized (classes) {
      String id = (String) classes.nextElement();
      return new GameInfo(GameFactory.getSharedInstance().createGame(id));
								     
								     

    }              
  }
  
}










