/*
 * $Id: GameLinkFilter.java,v 1.1 1998/03/11 02:10:17 vmallet Exp $
 *
 *
 * $Log: GameLinkFilter.java,v $
 * Revision 1.1  1998/03/11 02:10:17  vmallet
 * Initial revision
 *
 *
 */

package game;


import java.io.*;
import java.util.*;


/**
 * When a Jags client is connected to a Jags server, it can open multiple games
 * on this server. All those games will share the same <code>GameLink</code> object,
 * so we need a way to sort network messages addressed to different games. This is
 * done through the <code>GameLinkFilter</code> class.
 *
 * @version $Revision: 1.1 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public class GameLinkFilter implements GameLinkListener {

  /**
   * Creates a new GameLinkFilter [DESCRIBE ME]
   */
  public GameLinkFilter(Object id, GameLink gameLink) {
    this.gameLink = gameLink;
    
    gameId = id;

    gameLink.addGameLinkListener(this, gameId);
  }
  
  /**
   * Sends the given frame over the network. The frame will be tagged according
   * to the game id.
   *
   * @exception IOException thrown if an error occured during the sending 
   *            of the frame 
   * @see #sendBlindly
   * @see GameLink#sendBlindly
   */  
  public void send(LinkFrame frame) throws IOException {
    frame.setGameId(gameId);
    gameLink.send(frame);
  }
  
  /**
   * experimental.
   *
   * @see #send
   * @see GameLink#send
   */  
  public void sendBlindly(LinkFrame frame) {
    frame.setGameId(gameId);
    gameLink.sendBlindly(frame);
  }


  // == implements GameLinkListener ===========================

  /**
   * Messaged when a new data frame is received by the <code>GameLink</code>
   * object and this frame belongs to the groups the GameLinkListener
   * asked to listen to.
   *
   * @param frame the frame that has just been received.
   *
   * @see GameLink
   */  
  public void frameReceived(Object link, LinkFrame frame) {
    // simply retransmits to listeners
    notifyListeners(frame);
  }

  

  // == Listeners code ========================================

  /**
   * Adds the specified listener to the sets of listeners of this 
   * <code>GameLinkFilter</code> object. The listener will listen only to the 
   * messages related to the group.
   *
   * @param listener the specified listener to be added.
   * @param gameId   the group of messages the listener wants listen to.
   */
  public synchronized void addGameLinkListener(GameLinkListener listener, 
					       int group) {
    Integer groupInt = new Integer(group);
    Vector  groupListeners;

    if (listeners.containsKey(groupInt))
      groupListeners = (Vector) listeners.get(groupInt);
    else {
      groupListeners = new Vector();
      listeners.put(groupInt, groupListeners);
    }
    
    groupListeners.addElement(listener);
  }
  

  /**
   * Removes the specified listener from the sets of listeners of this 
   * <code>GameLink</code> object for the given group.
   * 
   * @param listener the listener to be removed from the set.
   * @param gameId   the given group.
   */
  public synchronized void deleteGameLinkListener(GameLinkListener listener, 
						  int group) {
    Vector groupListeners = (Vector) listeners.get(new Integer(group));
    //@@ we might want to do something if the group does not exist. -vm 980308
    if (groupListeners != null)
      groupListeners.removeElement(listener);
  }


  /**
   * experimental.
   */  
  void notifyListeners(LinkFrame frame) {
    Integer group = new Integer(frame.getGroup());
    Vector groupListeners = (Vector) listeners.get(group);

    if (groupListeners != null) {
      for (Enumeration f = groupListeners.elements(); f.hasMoreElements(); ) {
	GameLinkListener listener = (GameLinkListener) f.nextElement();
	listener.frameReceived(this, frame);
      }
    }
  }


  /**  
   * Set of listeners of this <code>GameLinkFilter</code> object.
   */  
  Hashtable listeners = new Hashtable();

  
  /**
   * The underlying <code>GameLink</code> object.
   */
  GameLink gameLink;

  /**
   * The id of the game this <code>GameLinkFilter</code> corresponds to.
   */
  Object gameId;

} // class GameLinkFilter
