/*
 * $Id: GameServer.java,v 1.2 1998/03/11 02:14:17 vmallet Exp $
 *
 *
 * $Log: GameServer.java,v $
 * Revision 1.2  1998/03/11 02:14:17  vmallet
 * Added the removePlayerFromGame() method.
 *
 * Revision 1.1  1998/03/02 19:28:31  vmallet
 * Initial revision
 *
 *
 */ 

package game;

import java.io.*; 
import java.util.*; 
import java.net.*; 

/**
 * The <B>GameServer</B> class is the main class of the server side
 * of the multi-game. It contains the <CODE>main</CODE> method which
 * is used to boot up the server.
 *
 * [FINISH ME]
 *
 * @version $Revision: 1.2 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public class GameServer implements HandlerListener, GameLinkConstants { 
  
  /**
   * Set of all games currently running.
   */  
  Hashtable gameHandlers = new Hashtable();
  

  /**
   * [DESCRIBE ME]
   */  
  GameFactory gameFactory;
  

  /**
   * The name of this GameServer.
   */
  String name;

  /**
   * I/O Port the server will listen to.
   */
  int port;

  /**
   * Properties for this GameServer.
   */  
  Properties props;

  /**
   * Current id attributed to new games.
   */
  Integer currentId = new Integer(0);

  /**
   * if true, enables the printing of debug informations on the console.
   */  
  boolean debug = true;

  

  /**
   * Creates a new GameServer
   */
  public GameServer() {
    props = new Properties();
    
    URL url = null;
    try {
      url = getClass().getResource("GameServer.properties");
      props.load(url.openStream());
    } catch (Exception e) {
      if (debug)
	System.out.println("<GameServer> warning: " + 
			   "failed to load properties: " + url);
      // we'll use defaults
    }
    
    name = props.getProperty("GameServer.name", "Unnamed Server");
    
    try {
      port = Integer.parseInt(props.getProperty("GameServer.port", "4213"));
    } catch (Exception e) {
      if (debug)
	System.out.println("<GameServer> Warning: invalid GameServer.port" +
			   " property. Using default value of 4213");
      port = 4213;
    }

    gameFactory = GameFactory.getSharedInstance();
    System.out.println("\nLoaded Games: ");
    for (Enumeration e = gameFactory.getGameInfos(); e.hasMoreElements(); ) {
      System.out.println( "   + " + (GameInfo) e.nextElement());
    }
  }  
  
  /**
   * Returns the name of this GameServer
   */  
  public String getName() {
    return name;
  }

  /**
   * Returns the I/O port the server will listen to.
   */  
  public int getPort() {
    return port;
  }


  /**
   * Returns a Vector of <code>GameInfo</code> informations for
   * all games in the server, including new and running games.
   */  
  public Vector getAllGameInfos() {
    Vector v = new Vector();
    
    // new games
    for (Enumeration e = gameFactory.getGameInfos(); e.hasMoreElements(); )
      v.addElement((GameInfo) e.nextElement());

    // running games
    //@@ check thread safeness ! -vm 980224
    for (Enumeration e = gameHandlers.elements(); e.hasMoreElements(); ) 
      v.addElement(new GameInfo((GameHandler) e.nextElement()));
    
    return v;
  }

  /**
   * Adds a new game in the server, creating the appropriate 
   * <code>GameServer</code>, with the specified game id.
   *
   * @param id the id that has to be associated with game
   * @param game the game to be added into the server
   * @return the new GameHandler created on top of game.
   */
  public GameHandler addGame(Game game) {
    GameHandler handler = new GameHandler(game, this);
    handler.setId(getNewId());
    
    LinkFrame frame = new LinkFrame();
    frame.setGameId(HANDLER_ID);
    frame.setGroup(HANDLER_GROUP);
    frame.setRequest(ST_GAME_ADDED, NONE);
    frame.setData(new GameInfo(handler));

    sendToClients(frame);

    synchronized(gameHandlers) {
      gameHandlers.put(handler.getId(), handler);
    }

    return handler;
  }
  
  /**
   * Returns the game corresponding to the given id. The game is either
   * a currently running game, in which case it is returned directly, 
   * or a new game. In this latter case the game is created and returned.
   * The object returned is actually a <code>GameHandler</code> which is
   * used to hold and run games in the server.
   *
   * @param id the id of the game requested.
   * @return the game retrieved or created, or <code>null</null> if the
   *         game cannot be found.
   */
  public GameHandler getGame(Object id) {
    if (id instanceof String) {
      Game game = gameFactory.createGame((String) id);
      if (game == null)
	return null;
      
      GameHandler handler = addGame(game);

      if (debug) 
	System.out.println("<GameServer> getGame(): created game: " + handler);

      return handler;
    } else if (id instanceof Integer) {
      return (GameHandler) gameHandlers.get(id);
    }

    return null;
  }


  /**
   * Remove the specified game from the set of games runned by the
   * server.
   *
   * @param handler the game to be removed from the server.
   */
  public void removeGame(GameHandler handler) {
    if (debug)
      System.out.println("<GameServer> removeGame(): " + handler);

    synchronized(gameHandlers) {
      gameHandlers.remove(handler.getId());
    }
    
    LinkFrame frame = new LinkFrame();
    frame.setGameId(HANDLER_ID);
    frame.setGroup(HANDLER_GROUP);
    frame.setRequest(ST_GAME_REMOVED, NONE);
    frame.setData(new GameInfo(handler));
    
    sendToClients(frame);
  }

   
  /**
   * Removes the specified player from the given game. If this makes the
   * game terminate, then remove the game from the server.
   *
   * @param player  the player to remove from the game
   * @param handler the <code>GameHandler</code> from which to remove the player.
   */
  public void removePlayerFromGame(Player player, GameHandler handler) {
    LinkFrame frame = new LinkFrame();
    frame.setGameId(HANDLER_ID);
    frame.setGroup(HANDLER_GROUP);
    frame.setRequest(ST_INFO_CHANGED, NONE);
    
    Game game = handler.getGame();
    
    game.removePlayer(player);
    
    //@@ quick'n dirty. check that. -vm980301
    if (game.isTerminated()) {
      removeGame(handler);
    } else {
      frame.setData(new GameInfo(handler));
      sendToClients(frame);
    }
  }



  /**
   * Returns a new unique Id for a new game beeing created.
   */  
  public Integer getNewId() {
    Integer newId = currentId;
    currentId = new Integer(newId.intValue() + 1);
    return newId;
  }
 
  /** @@ not really sure this should be here */
  public void handlerHasChanged(Handler handler) { } 
  /** @@ not really sure this should be here */
  public void handlerLostConnection(Handler handler) { 
    
  }


  /**
   * Sends the given frame to all currently connected clients.
   * The lastly added client will be messaged first.
   *
   * @param frame the frame to send to all clients.
   */
  public void sendToClients(LinkFrame frame) {
    ClientHandler[] handlers;
    int size = 0;

    synchronized(clientHandlers) {
      size = clientHandlers.size();
      handlers= new ClientHandler[size];
      clientHandlers.copyInto(handlers);
    }

    for (int i = size - 1; i >= 0; i--) 
      handlers[i].getGameLink().sendBlindly(frame);
  }
  








  /**
   * The GameServer entry point. [FINISH ME]
   * 
   */
  public static void main(String args[]) { 
    GameServer theServer = new GameServer();

    System.out.println("Launching server '" + theServer.getName() 
		       + "` on port " + theServer.getPort() + "...");
    theServer.waitForConnections();
  }
  
  /**
   * [DESCRIBE ME] 
   */
  public void waitForConnections() {
    // truc de base ultra a l'arrache, juste pour me clarifier les idees
    
    ServerSocket serverSocket = null;
    
    try {
      serverSocket = new ServerSocket(port);
    } catch (IOException e) {
      System.out.println("Bleme sur creation socket");
      System.exit(0);
    }
    
    Socket socket = null;
    
    while (!shouldQuit) {
      
      try {
	socket = serverSocket.accept();
      } catch (IOException e) { //@@ faire gestion propre 
	System.out.println("bleme sur accept!");
	System.exit(0);
      }
      
      
      System.out.println("Connexion reussie sur port: " 
			 + socket.getPort()); 
    
      ClientHandler handler;
      try {
	handler = new ClientHandler(this, socket);
      } catch (SocketException e) {
	handler = null;
      }
      
      if (handler != null) {
	handler.start();
	synchronized(clientHandlers) {
	  clientHandlers.addElement(handler); // handler will quit alone.
	}
      }
    }
    
    try {
      serverSocket.close();
    } catch (Exception e) { }
    
  }


  /**
   * Removes the specified <code>ClientHandler</code> from the set of
   * running client handlers.
   *
   * @param handler the handler to remove from the set.
   */  
  public void removeHandler(ClientHandler handler) {
    synchronized(clientHandlers) {
      clientHandlers.removeElement(handler);
    }
  }
  

  // Private Area

  /** [DESCRIBE ME]  */    
  boolean shouldQuit = false;

  

  /** 
   * List of Clients currently connected.
   */  
  Vector clientHandlers = new Vector();
}





