/*
 * $Id: LinkFrame.java,v 1.3 1998/04/06 18:30:44 vmallet Exp $
 *
 *
 * $Log: LinkFrame.java,v $
 * Revision 1.3  1998/04/06 18:30:44  vmallet
 * Added the setSubRequest() method.
 *
 * Revision 1.2  1998/03/11 02:01:11  vmallet
 * Added the game Id code.
 * Did some documentation.
 *
 * Revision 1.1  1998/03/02 19:28:31  vmallet
 * Initial revision
 *
 *
 */

package game;

import java.io.*; 

/**
 * The <code>LinkFrame</code> class is a placeholder for data to be 
 * exchanged over the GameLink data link. 
 * <p>A frame object would typically be used this way :
 * <pre>
 *   LinkFrame frame = new LinkFrame(); // construct the frame object
 *   frame.setGroup(YOUR_FRAME_GROUP_HERE); // type the frame
 *   frame.setRequest(YOUR_REQUEST, YOUR_SUB_REQUEST); // set the Request
 *   frame.setData(theDataObject); // put the data in the frame
 *   // send the frame through the GameLink
 * </pre>
 *
 * @see GameLink
 * 
 * @version $Revision: 1.3 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public class LinkFrame implements Serializable {

  /**
   * Creates a new LinkFrame.
   */
  public LinkFrame() {
    currentSize = 0;
  }

  /**
   * Sets the game id this frame has been created for. This should be automatically
   * done by the <code>GameLinkFilter</code> object.
   */
  public void setGameId(Object id) {
    gameId = id;
  }

  /**
   * Returns the game id this frame has been created for.
   */
  public Object getGameId() {
    return gameId;
  }

  /**
   * Sets the group the frame belongs to. The frame group is used to
   * sort the frames when they're received.
   *
   * @param group the frame group 
   * @see GameLinkConstants
   */  
  public void setGroup(int group) {
    this.group = group;
  }

  /**
   * Returns the group the frame belongs to. 
   *
   * @see #setGroup
   */  
  public int getGroup() {
    return group;
  }
  
  /**
   * Sets the request and the subrequest carried by the frame. The
   * request allows the receiver of the frame to know how to deal
   * with the data contained in the frame.
   * 
   * @param request the request of this frame. 
   * @param subRequest the sub-request of the request. Usually 
   *                   <code>NONE</code>
   * @see GameLinkConstants
   */
  public void setRequest(int request, int subRequest) {
    this.request    = request;
    this.subRequest = subRequest;
  }

  /**
   * Sets the subRequest carried by the frame. 
   * 
   * @param subRequest the sub-request of the request. Usually 
   *                   <code>NONE</code>
   * @see #setRequest
   * @see GameLinkConstants
   */
  public void setSubRequest(int subRequest) {
    this.subRequest = subRequest;
  }

  /**
   * Returns the request contained in the frame.
   */  
  public int getRequest() {
    return request;
  }

  /**
   * Returns the sub-request contained in the frame.
   */  
  public int getSubRequest() {
    return subRequest;
  }

  
  /**
   * Puts new data in the frame. The data is a single object, so it has to
   * be a container if you need to carry more than one object. The data 
   * object also has to be <code>Serializable</code>, as it will be
   * transmitted as-is on the network. (Note that this may change in the
   * future for performance cosiderations).
   *
   * @param data the data Object, which is necessarily 
   *             <code>Serializable</code>
   * @see java.util.Serializable
   */  
  public void setData(Object data) {
    this.data = data;
  }
  
  /**
   * Returns the data contained in this frame.
   *
   * @see #setData
   */  
  public Object getData() {
    return data;
  }


  /**
   * Returns a string representation of the object.
   */
  public String toString() {
    return "" + group + ":<" + request + "," + subRequest + "> - " + data;
  }


  /** Id of the game this frame has been created for */
  Object gameId;

  /** [DESCRIBE ME]  */  
  int group;

  /** [DESCRIBE ME]  */  
  int request; 

  /** [DESCRIBE ME]  */  
  int subRequest; 

  /** [DESCRIBE ME]  */  
  Object data;

  /** [DESCRIBE ME]  */  
  transient int  currentSize;

}

