/*
 * $Id: Player.java,v 1.5 1998/04/07 12:18:22 vmallet Exp $
 *
 *
 * $Log: Player.java,v $
 * Revision 1.5  1998/04/07 12:18:22  vmallet
 * More debug output.
 *
 * Revision 1.4  1998/04/06 19:39:32  vmallet
 * Changed constructor Player(frameData) to Player(id, frameData).
 * Fixed a few bugs in constructors.
 * Readded the Player(name) constructor.
 *
 * Revision 1.3  1998/04/05 19:47:09  lozac
 * Added getFrameData() code and constructors.
 *
 * Revision 1.2  1998/04/05 13:49:30  lozac
 * GUI code (getComponent)
 *
 * Revision 1.1  1998/04/04 12:01:04  lozac
 * Initial revision
 *
 *
 */ 

package game;
import scrobble.*;
import java.util.*; 
import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;


/**
 * The <code>Player</code> class manages the player in a general way, so it 
 * can be used in many games.
 *
 * @author  Kristell Lozac'hmeur (lozac@email.enst.fr)
 * @version $Revision: 1.5 $
 */

public class Player implements Observer, GameLinkConstants, GameLinkListener {

  /**
   * Name of Player
   */ 
  protected  String name;
	

   /**
    *  Player's score.
   */
  protected int score;

  /**
   * The identity of the last Player 
   */
  static int generalId;

  /** 
   * Player's identity
   */
  int id;

  /**
   * to know if the data inform about the Player.
   */  
  protected int PERSO_DATA = 10;

  /**
   * to know if the data inform about an other Player.
   */  
  protected int OTHER_DATA = 11;

  /**
   * Player uses an object network for the commnication client/Server.
   */
  protected GameLinkFilter gamelink;

  /**
   * Player must be Observable by the others Players. 
   * So needs the object o to have the methods addObserver(), deleteObserver() 
   * and notifyObserver().
   */
  protected Observable o;

  /**
   * The label which contains the name of Player;
   */
  protected JLabel nameLabel;
  
  /**
   * The label which contains the score of Player;
   */
  protected JLabel scoreLabel;

  
  /**
   * The panel which represents the Player (with his name and his score);
   */
  protected JPanel playerPanel;
  

  // Constructor

  /**
   * Creates a Player without parameters
   */
  public Player() {
    id = getNewId(); 
    o = new Observable() {
      public void notifyObservers(Object o) {
	setChanged();
	super.notifyObservers(o);
      }
    };
    
  }

  
  /**
   * Creates a new Player with only his name.
   */
  public Player(String name) {
    this();
    this.name = name;
  }
  
  /**
   * Creates a new Player with the informations contained in a 
   * object. This Player will exit only in the client so he doesn't need to be observable. 
   * @param frameData the object which contains the informations about the Player.
   */
  public Player(int id, Object frameData) {
    Object[] frameArray = (Object[]) frameData;

    this.id    = id;
    this.name  = (String) frameArray[1];
    this.score = ((Integer) frameArray[2]).intValue();

    System.out.println("<Player> init(id, frameData): " + this);

  }
  
   
  /**
   * Creates a new Player with his name and his gamelink.
   */
  public Player(String name, GameLinkFilter gamelink) {
    this();
    this.name = name;
    this.gamelink = gamelink;
  }

   /**
   * Creates a new Player with his name and his identity.
   * @param name the Player's name.
   * @param id the identity of Player.
   */
  public Player(String name, int id) {
    this();
    this.name = name;
    this.id = id;     

    System.out.println("<Player> init(name,id): " + this);
  }

  // Methods

  /**
   * This method is called when a Player is created. 
   * It gives to this new Player his identity.
   * @return the Player's new identity;
   */  

  public synchronized int getNewId() {
    int newId;

    generalId++;
    newId = generalId;
    return newId;
  } 

  /**
   * Return the Player's identity 
   * @return the Player's identity 
   */
  public int getId() {
    return id;
  }


  /**
   * Returns the Player's score.
   * @return the Player's score.
   */
  public int getScore() {
    return score;
  }
  
  /**
   * Set a new Player 's score.
   * @param  newscore the new score of Player.   
   */
  public void setScore(int newscore) {
    LinkFrame frame;

    score = newscore;
    frame = createLinkframe(RQ_PLAYERNEWSCORE, new Integer(score), getId());
    System.out.println("<Player> : nb d'observers = " + countObservers());
    notifyObservers(frame);
  }

  /**
   * Returns the Name of Player.
   * @return the Player's Name. 
   */
  public String getName() {
    return name;
  }

  
  /**
   * Set the name of Player.
   * @param newName the new Player's name.
   */
  public void setName(String newname) {
    LinkFrame frame;

    name = newname;
    frame = createLinkframe(RQ_PLAYERCHANGENAME, name, getId());
    notifyObservers(frame);
  }

  /**
   * Returns the GameLinkFilter of Player.
   * @return the Player's gamelink.
   */
  public GameLinkFilter getGameLink() {
    return gamelink; 
  }

  /**
   * Set the GameLinkFilter of Player.
   * @param newGameLink the new Player's gamelink.
   */  
  public void setGameLink(GameLinkFilter newgamelink) {
    gamelink = newgamelink;
  }

  /**
   * This method sends the linkframe received from an observed object.
   * @param observable the observed object.
   * @param linkframe the Object received from the observed object.
   */
  public void update(Observable object, Object linkframe) {
    gamelink.sendBlindly((LinkFrame) linkframe);
  }

  

  /**
   * Tells if Player is a ComputerPlayer or not. 
   * This method must be implemented whenever a <code>ComputerPlayer</code> class will be created.
   * @return true if the Player is a ComputerPlayer, false otherwise
   */
  public boolean isComputer() {
    return false;
  }

  /**
   * Adds an observer to the set of observers for this object.
   * @param obs an observer to be added.
   */
  public void addObserver(Observer obs) {
    o.addObserver(obs);
  }

  /**
   * Deletes an observer to the set of observers for this object.
   * @param obs the observer to be deleted
   */
  public void deleteObserver(Observer obs) {
    o.deleteObserver(obs);
  }

  /**
   * Clears the observerlist so that this object no longer has any observers.
   */
  public void deleteObservers() {
    o.deleteObservers();
  }

  /**
   * Notifies all the observers of this Player that he has changed and indicates this 
   * changement by a frame that will be send by the method update on a GameLinkFilter.
   * @param frame the Linkframe that indicates the changement.
   */
  public void notifyObservers(LinkFrame frame) {
    o.notifyObservers(frame);
  }


  /**
   * Returns the number of observers of this object.
   * @return the number of observers of this object.
   */  
  public int countObservers() {
    return o.countObservers();
  }

   /**
   * Invoked when a new data frame is received.
   * @param frame the frame that has just been received.
   */  
  public void frameReceived(Object link, LinkFrame frame) {
    int request = frame.getRequest();
    
    if (frame.getSubRequest() == id) {
      switch (request) {
      case RQ_PLAYERCHANGENAME :
	name = (String) frame.getData(); 
	nameLabel.setText(getName());
	break;
      case RQ_PLAYERNEWSCORE :  
	score = ((Integer) frame.getData()).intValue();
	scoreLabel.setText(" - score : " + getScore());
	break;
      default : 
	System.out.println("<Player> frameReceived(): Unhandled request");  
	break;
      }
    }
  }

  
  /**
   * Creates a new LinkFrame, which concerns only one Player. 
   * @param request the request of the frame.
   * @param data the data of the frame.
   * @param playerid the identity of the Player which is concerned by the request.
   */  
  public LinkFrame createLinkframe(int request, Object data, int playerid) {
    LinkFrame frame = new LinkFrame();
    
    frame.setGroup(PLAYER_GROUP);
    frame.setRequest(request, playerid);
    frame.setData(data);

    return frame;  
  }

  /**
   * Helper function for getFrameData() and getFrameDataPerso()
   * @return the object which contains the informations about the player.
   */
  private Object getFrameData0() {
    Object[] frameData = new Object[10];
    
    frameData[0] = new Integer(0);
    frameData[1] = getName();
    frameData[2] = new Integer(getScore());

    return frameData;
  }


  /**
   * Returns an object which contains the informations necessaries for the 
   * constructions by a new client of a player, which still exists in the server.
   * @return the object which contains the informations about the player.
   */  
  public Object getFrameData() {
    Object[] frame = new Object[10];   
    frame = (Object[]) getFrameData0();
    frame[0] = new Integer(OTHER_DATA);
    return frame;
  }

  /**
   * Returns an object which contains the informations necessaries for the 
   * constructions by a new client of its own player.
   * @return the object which contains the informations about the player.
   */  
  public Object getFrameDataPerso() {
    Object[] frame = new Object[10];   

    frame = (Object[]) getFrameData0();
    frame[0] = new Integer(PERSO_DATA);
    return frame;
  }



  /**
   * Returns a string representation of the object.
   */
  public String toString() { 
    return getName() + " " + getId() + " : " + getScore();
  }

  /**
   * Returns a Panel containing the name, and the score of the Player
   */  
  public Component getComponent() {
    if (playerPanel == null) {
      nameLabel = new JLabel(getName());
      scoreLabel = new JLabel(" - score : " + getScore());
      playerPanel = new JPanel();
      playerPanel.setLayout(new BoxLayout(playerPanel, BoxLayout.X_AXIS));
      playerPanel.add(nameLabel);
      playerPanel.add(scoreLabel);
    }
    return playerPanel;
    
  }
  

  public static void main(String args[]) {
    Player player1 = new Player("kristell");
    
    player1.setScore(25);
    System.out.println("le joueur " + player1.getName() + " a " + player1.getScore() + " points.");
    System.out.println("Son identite est :"+ player1.getId());
    System.out.println("le joueur " + player1.getName() + " a " + player1.countObservers() + " observers ");  

    Player player2 = new Player("Charlie");
    player1.addObserver(player2);
    player2.addObserver(player1);
    System.out.println("creation d'un nouveau joueur : " + player2.getName()  );
    System.out.println("Son identite est :"+ player2.getId());
    System.out.println("le joueur " + player1.getName() + " a " + player1.countObservers() + " observers (c'est " + player2.getName() + " ) ");  
    System.out.println("le joueur " + player2.getName() + " a " + player2.countObservers() + " observers (c'est " + player1.getName() + " ) ");  

    System.out.println("le joueur " + player1.getName() + " change de nom : " );
    player1.setName("Annette");

  }

}  















