/*
 * $Id: ServerHandler.java,v 1.3 1998/04/06 09:23:22 vmallet Exp $
 *
 *
 * $Log: ServerHandler.java,v $
 * Revision 1.3  1998/04/06 09:23:22  vmallet
 * Removed the Player creation code (now done in ClientGame).
 *
 * Revision 1.2  1998/03/11 02:17:20  vmallet
 * Did a lot of requests handling code.
 *
 * Revision 1.1  1998/03/02 19:28:31  vmallet
 * Initial revision
 *
 *
 */

package game;

import java.io.*; 
import java.net.*;
import java.util.*;


/**
 * The <code>ServerHandler</code> class handles the server connection
 * in the client. [DESCRIBE ME MORE]
 * 
 * @version $Revision: 1.3 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public class ServerHandler extends Handler {
  
  /**
   * DNS name of the server we're dealing with (or IP address)
   */
  String address;

  /**
   * Name of the server (retrieved from the server itself)
   */
  String name;

  /**
   * Port number of the server
   */
  int port;

  /**
   * Version of the server
   */ 
  int version;

  /**
   * Vector containing GameInfos about the games the server can provide.
   */  
  Vector gameInfos;


  // Constructor

  /**
   * Creates a new ServerHandler.
   *
   * @param socket the socket connected to the server.
   * @param address name or IP address of the server.
   * @param port the port the server listens to.
   */
  public ServerHandler(Socket socket,
		      String address, int port) throws SocketException {   
    super(socket);

    this.address = address;
    this.port = port;
  }



  // Methods

  /**
   * Starts this server handler. This methods keeps on executing until the connection
   * with server is lost, so it should be run in a separate thread.
   */  
  public void run() {
    LinkFrame frame = new LinkFrame();
    Object[]  data  = new Object[5];

    data[0] = new Integer(0);  //@@ Client version number [CHECK ME]
    data[1] = "Java";          //@@ Client language [CHECK ME]

    // When connecting to a server, ask for this server Id.
    frame.setGameId(HANDLER_ID);
    frame.setGroup(HANDLER_GROUP);
    frame.setRequest(RQ_SERVERID, NONE);
    frame.setData(data);

    gameLink.sendBlindly(frame);

    while (gameLink.isConnected()) {
      try { sleep(1000); } catch (InterruptedException e) { }
      // here we could do important handler stuff.
    }
    
    if (debug)
      System.out.println("<ServerHandler> run() - Lost Server Connection!");
    
    lostConnectionNotify();
  }

  /**
   * Returns the host name or IP address of the server we're connected to
   */  
  public String getServerAddress() {
    return address;
  }

  /**
   * Returns the name of the server we're connected to
   */  
  public String getServerName() {
    return name;
  }

  /**
   * Returns the port of the server we're connected to
   */  
  public int getServerPort() {
    return port;
  }

  /**
   * Returns the number of games available from this server
   */
  public int countGames() {
    return gameInfos.size();
  }
  
  /**
   * Returns the <code>GameInfo</code> corresponding to the given Id.
   *
   * @param id the id used to search for the <code>GameInfo</code>
   * @return the corresponding <code>GameInfo</code>
   */  
  public GameInfo getGameInfo(Object id) {
    GameInfo info = null;
    for (Enumeration e = getGameInfos(); 
	 info == null && e.hasMoreElements();) {
      GameInfo tmpInfo = (GameInfo) e.nextElement();
      if (tmpInfo.getGameId().equals(id))
	info = tmpInfo;
    }

    return info;
  }

  
  /**
   * Returns all <code>GameInfo</code> available from this server.
   */
  public Enumeration getGameInfos() {
    return gameInfos.elements();
  }
    
  /**
   * Removes from the game infos set the <code>GameInfo</code> corresponding
   * to the given Id. In this very particular case, id may be a regular
   * game id OR a GameInfo object.
   *
   * @param id the id of the game to be removed from the client.
   */
  public void removeGameInfo(Object id) {
    GameInfo info;

    if (id instanceof GameInfo)
      info = (GameInfo) id;
    else
      info = getGameInfo(id);

    synchronized(gameInfos) {
      gameInfos.removeElement(info);
    }
  }

  /**
   * Returns a string representation of the object.
   */
  public String toString() {
    if (getServerName() != null) 
      return pad(getServerName(), 50) + 
	pad(getServerAddress() + " : " + getServerPort(), 20) +
	"" + countGames() + " game" + ((countGames() > 1) ? "s" : " ");

    return pad(getServerAddress(), 15) + " : " + getServerPort();
  }


  /**
   * Messaged when a new data frame is received by the <code>GameLink</code>
   * object and this frame belongs to the groups the GameLinkListener
   * asked to listen to.
   *
   * @param frame the frame that has just been received.
   *
   * @see GameLink
   */  
  public void frameReceived(Object link, LinkFrame frame) {
    int request = frame.getRequest();

    LinkFrame answer = new LinkFrame();    
    answer.setGameId(HANDLER_ID);
    answer.setGroup(HANDLER_GROUP);

    switch (request) {
    case RQ_SERVERID:
      if (debug)
	System.out.println("<ServerHandler> frameReceived() - got RQ_SERVERID");
      Object[] data = (Object[]) frame.getData();
      version = ((Integer) data[0]).intValue(); //@@ server version. CHECK THIS
      name      = (String) data[1]; // server name. 
      gameInfos = (Vector) data[2]; // infos about all games

      setChangeStatus(BIG_CHANGE);
      notifyListeners();
      
      if (debug) {
	System.out.println("   + name   : " + name);
	System.out.println("   + version: " + version);
	System.out.println("   + games  : " + gameInfos.size());
      }
      break;

    case ST_GAME_ADDED:
      {
	if (debug)
	  System.out.println("<ServerHandler> frameReceived(): got ST_GAME_ADDED");
	GameInfo info = (GameInfo) frame.getData();
	gameInfos.addElement(info);
	//      nbGames   = gameInfos.size(); // number of available games. 
	
	setChangeData(info);
	setChangeStatus(INFO_ADDED);
	notifyListeners();
      }
      break;
      
    case ST_CREATE_GAME: 
      System.out.println("<ServerHandler> ST_CREATE_GAME");
      doCreateGame(frame);
      break;

    case ST_GAME_REMOVED:
      if (debug) 
	System.out.println("<ServerHandler> frameReceived(): got ST_GAME_REMOVED");
      doGameRemoved(frame);
      break;

    case ST_INFO_CHANGED:
      if (debug)
	System.out.println("<ServerHandler> frameReceived(): got ST_INFO_CHANGED");
      doInfoChanged(frame);
      break;
      
    default:
      if (debug)
	System.out.println("<ServerHandler> warning: unhandled request" + request);
      break;
    }
  } // frameReceived

  
  /**
   * Handles the ST_INFO_CHANGED message.
   *
   * @param frame the frame containing the ST_INFO_CHANGED message.
   */
  public void doInfoChanged(LinkFrame frame) {
    GameInfo info  = (GameInfo) frame.getData();
    GameInfo other = getGameInfo(info.getGameId());
    
    if (other == null) {
      if (debug)
	System.out.println("<ServerHandler> ST_INFO_CHANGED: game error");
      return;
    }
    
    other.update(info);
    
    setChangeData(info);
    setChangeStatus(INFO_ADDED);
    notifyListeners();
  }


  /**
   * Handles the ST_GAME_REMOVED message.
   *
   * @param frame the frame containing the ST_GAME_REMOVED message.
   */
  public void doGameRemoved(LinkFrame frame) {
    GameInfo info = (GameInfo) frame.getData();

    GameInfo removedInfo = getGameInfo(info.getGameId());
    
    removeGameInfo(removedInfo);
    
    setChangeData(removedInfo);
    setChangeStatus(INFO_REMOVED);
    notifyListeners();
  }
  


  /**
   * Handles the ST_CREATE_GAME message. Actually instantiates the client game
   * class and start the client game.
   *
   * @param frame the frame containing the ST_CREATE_GAME message.
   */
  public void doCreateGame(LinkFrame frame) {
    Object[]   data = (Object[]) frame.getData();
    ClientGame game = null;
    
    try {
      game = (ClientGame) Class.forName((String) data[0]).newInstance();
    } catch (ClassNotFoundException e) {
      if (debug) 
	System.out.println("<ServerHandler> class not found: " + data[0]);
      //@@ create apropriate dialog box for notifying user.
    } catch (InstantiationException e) {
      if (debug) 
	System.out.println("<ServerHandler> instantiation error: " + data[0]);
      //@@ create apropriate dialog box for notifying user.
    } catch (IllegalAccessException e) {
      if (debug) 
	System.out.println("<ServerHandler> instantiation error: " + data[0]);
      //@@ create apropriate dialog box for notifying user.
    }
    //@@ check how to make the previous catch code cleaner.

    int playerId =  ((Integer) data[3]).intValue();
    game.setId(data[1]);
    game.setName((String) data[4]);
    game.make(new GameLinkFilter(game.getId(), gameLink), playerId); // initialize. 
    
    //@@ and create the first player (our player).

    // code removed 1998.04.05 - Games should handle player creation themselves.
    //     Player player = game.createPlayer((String) data[2], playerId);
    //     game.addPlayer(player);
  }
  
  /**
   * Returns the last change status of this handler.
   */  
  public int getChangeStatus() {
    return changeStatus;
  }


  /**
   * 
   */
  void setChangeStatus(int status) {
    changeStatus = status;
  }
  
  /**
   * 
   */  
  public void clearChanged() {
    changeStatus = NO_CHANGE;
    changeData   = null;
  }

  /**
   *
   */
  void setChangeData(Object data) {
    changeData = data;
  }

  /**
   *
   */
  public Enumeration getChangeInfos() {
    return ((Vector) changeData).elements();
  }
  
  /**
   *
   */
  public GameInfo getChangeInfo() {
    return (GameInfo) changeData;
  }
  

  /**   */  
  Object changeData;
  
  /**   */  
  int changeStatus = NO_CHANGE;

  /**   */  
  public final static int NO_CHANGE    = 0;

  /**   */
  public final static int BIG_CHANGE   = 10;

  /**   */  
  public final static int INFO_CHANGED = 20;

  /**   */  
  public final static int INFO_ADDED   = 30;

  /**   */
  public final static int INFO_REMOVED = 40;

} // ServerHandler






